/*####################################################################
 FILE: TCN75A.cpp
 VERSION: 1.0
 PURPOSE: Digital Temperature Sensor with TCN75A library for Arduino

 HISTORY:
 Mirko Prosseda - Original version (20/11/2012)
#######################################################################*/


#include "TCN75A.h"
#include <Wire.h>   // library used with I2C protocol

#define temperature_reg 0x00
#define config_reg 0x01
#define temp_hyst_reg 0x02
#define temp_limitreg 0x03


void TCN75A::begin()
{
    Wire.begin();       // initializes I2C port
    baseaddress = B1001000;
    address = baseaddress;
}


void TCN75A::select_register(uint8_t reg)
{
    Wire.beginTransmission(address);
    Wire.write(reg);
    Wire.endTransmission();
}


void TCN75A::write_register(uint8_t reg, uint8_t byte)
{
    Wire.beginTransmission(address);
    Wire.write(reg);
    Wire.write(byte);
    if(reg == temp_hyst_reg || reg == temp_limitreg)
        Wire.write(0);
    Wire.endTransmission();
}


int TCN75A::read_register(uint8_t reg, uint8_t bytes)
{
    int w_reg = 0;

    // selects register to read
    select_register(reg);

    // receives data from selected register
    Wire.requestFrom(address, bytes);
    if(bytes == 2) {
        w_reg = Wire.read() << 8;
    }
    w_reg |= Wire.read();
    return(w_reg);
}


byte TCN75A::set_address(uint8_t add) // set the I2C sensor address as B1001XXX
{
    if(add < 8 && add >= 0)
    {
         address = baseaddress + add;
         return(0); // valid value (0 to 7)
    }
    else
         return(1); // invalid value
}


uint8_t TCN75A::set_resolution(uint8_t res) // set the measurement resolution as (9+res) bits
{
    uint8_t w_reg;

    if(res < 4 && res >= 0)
    {
        // selects Configuration Register
        select_register(config_reg);

        // calculates new value for Configuration Register
        Wire.requestFrom(address, byte(1)); // requires 1 byte from the sensor
        w_reg = Wire.read() & 0x9F;
        res = res << 5;
        w_reg |= res;

        // writes new value in Configuration Register
        write_register(config_reg, w_reg);
        return(0); // valid value (0 to 3)
    }
    else
        return(1); // invalid value
}


float TCN75A::read() // return ambient temperature as a float
{
    int tempreg = 0;         // temperature datas from the sensor

    // selects register of the measured temperature
    select_register(temperature_reg);

    // receives data from Temperature Register
    Wire.requestFrom(address, byte(2)); // requires 2 bytes from the sensor
    tempreg = Wire.read();              // first byte from Temperature Register
    tempreg = tempreg << 8;             // 8 positions shift to the left

    tempreg |= Wire.read();             // second byte from Temperature Register
    tempreg = tempreg >> 4;             // 4 positions shift to the right

    // calculates ambient temperature
    temperature = (float)tempreg / 16;  // Temperature Register divided by 16
    return(temperature);
}


void TCN75A::shutdown() // put the device in shut-down mode
{
    uint8_t w_reg;

    // selects Configuration Register
    select_register(config_reg);

    // calculates new value for Configuration Register
    Wire.requestFrom(address, byte(1)); // requires 1 byte from the sensor
    w_reg = Wire.read() | 0x01;

    // writes new value in Configuration Register
    write_register(config_reg, w_reg);
}


void TCN75A::turnon() // resume the device from shut-down mode
{
    uint8_t w_reg;

    // selects Configuration Register
    select_register(config_reg);

    // calculates new value for Configuration Register
    Wire.requestFrom(address, byte(1)); // requires 1 byte from the sensor
    w_reg = Wire.read() & 0xFE;

    // writes new value in Configuration Register
    write_register(config_reg, w_reg);
}


void TCN75A::enable_oneshot() // if the device is in shut-down mode will do one conversion and will go back in shut-down mode
{
    uint8_t w_reg;

    // selects Configuration Register
    select_register(config_reg);

    // calculates new value for Configuration Register
    Wire.requestFrom(address, byte(1)); // requires 1 byte from the sensor
    w_reg = Wire.read() | 0x80;

    // writes new value in Configuration Register
    write_register(config_reg, w_reg);
}


uint8_t TCN75A::set_fault_queue(uint8_t f)
{
    uint8_t w_reg;

    if(f < 4 && f >= 0)
    {
        // selects Configuration Register
        select_register(config_reg);

        // calculates new value for Configuration Register
        Wire.requestFrom(address, byte(1)); // requires 1 byte from the sensor
        w_reg = Wire.read() & 0xE7;
        f = f << 3;
        w_reg |= f;

        // writes new value in Configuration Register
        write_register(config_reg, w_reg);
        return(0); // valid value (0 to 3)
    }
    else
        return(1); // invalid value
}


uint8_t TCN75A::set_alert_polarity(uint8_t p)
{
    uint8_t w_reg;

    if(p == 0 || p == 1)
    {
        // selects Configuration Register
        select_register(config_reg);

        // calculates new value for Configuration Register
        Wire.requestFrom(address, byte(1)); // requires 1 byte from the sensor
        w_reg = Wire.read() & 0xFB;
        p = p << 2;
        w_reg |= p;

        // writes new value in Configuration Register
        write_register(config_reg, w_reg);
        return(0); // valid value (0 or 1)
    }
    else
        return(1); // invalid value
}


uint8_t TCN75A::set_comp_int(uint8_t b)
{
    uint8_t w_reg;

    if(b == 0 || b == 1)
    {
        // selects Configuration Register
        select_register(config_reg);

        // calculates new value for Configuration Register
        Wire.requestFrom(address, byte(1)); // requires 1 byte from the sensor
        w_reg = Wire.read() & 0xFD;
        b = b << 1;
        w_reg |= b;

        // writes new value in Configuration Register
        write_register(config_reg, w_reg);
        return(0); // valid value (0 or 1)
    }
    else
        return(1); // invalid value
}


void TCN75A::set_hysteresis(uint8_t h) // set the Temperature Hysteresis register
{
    // selects Temperature Limit-Set Register
    select_register(temp_hyst_reg);

    // writes new value in Temperature Hysteresis Register
    write_register(temp_hyst_reg, h);
}


void TCN75A::set_templimit(uint8_t l) // set the Temperature Limit-Set register
{
    // selects Temperature Limit-Set Register
    select_register(temp_limitreg);

    // writes new value in Temperature Limit-Set Register
    write_register(temp_limitreg, l);
}
